/**
 * Copyright (c) 2000-2010 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.security.auth;

import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.SetUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.permission.PortletPermissionUtil;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.util.PropsUtil;
import com.liferay.util.PwdGenerator;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * <a href="SessionAuthToken.java.html"><b><i>View Source</i></b></a>
 *
 * @author Amos Fong
 */
public class SessionAuthToken implements AuthToken {

	public SessionAuthToken() {
		if (PropsUtil.get("auth.token.ignore.actions") != null) {
			_ignoreActions = SetUtil.fromArray(
				PropsUtil.getArray("auth.token.ignore.actions"));
		}
		else {
			_ignoreActions = SetUtil.fromArray(_defaultIgnoreActions);
		}
	}

	public void check(HttpServletRequest request) throws PrincipalException {
		if (isIgnoreAction(request)) {
			return;
		}

		String requestAuthenticationToken = ParamUtil.getString(
			request, "p_auth");

		String sessionAuthenticationToken = getSessionAuthenticationToken(
			request, _PORTAL);

		if (!requestAuthenticationToken.equals(sessionAuthenticationToken)) {
			throw new PrincipalException("Invalid authentication token");
		}
	}

	public String getToken(HttpServletRequest request) {
		return getSessionAuthenticationToken(request, _PORTAL);
	}

	public String getToken(
		HttpServletRequest request, long plid, String portletId) {

		return getSessionAuthenticationToken(
			request, PortletPermissionUtil.getPrimaryKey(plid, portletId));
	}

	protected String getSessionAuthenticationToken(
		HttpServletRequest request, String key) {

		Map<String, String> sessionAuthenticationTokensMap =
			getSessionAuthenticationTokensMap(request);

		String sessionAuthenticationToken = sessionAuthenticationTokensMap.get(
			key);

		if (Validator.isNull(sessionAuthenticationToken)) {
			sessionAuthenticationToken = PwdGenerator.getPassword();

			sessionAuthenticationTokensMap.put(key, sessionAuthenticationToken);
		}

		return sessionAuthenticationToken;
	}
	
	protected Map<String, String> getSessionAuthenticationTokensMap(
		HttpServletRequest request) {

		HttpSession session = request.getSession();

		Map<String, String> sessionAuthenticationTokensMap =
			(Map<String, String>)session.getAttribute(
				"LIFERAY_SHARED_AUTHENTICATION_TOKEN");

		if (sessionAuthenticationTokensMap == null) {
			sessionAuthenticationTokensMap = new HashMap<String, String>();

			session.setAttribute(
				"LIFERAY_SHARED_AUTHENTICATION_TOKEN", sessionAuthenticationTokensMap);
		}

		return sessionAuthenticationTokensMap;
	}

	protected boolean isIgnoreAction(HttpServletRequest request) {
		String ppid = ParamUtil.getString(request, "p_p_id");

		String portletNamespace = PortalUtil.getPortletNamespace(ppid);

		String strutsAction = ParamUtil.getString(
			request, portletNamespace + "struts_action");

		return isIgnoreAction(strutsAction);
	}

	protected boolean isIgnoreAction(String strutsAction) {
		return _ignoreActions.contains(strutsAction);
	}

	private static final String[] _defaultIgnoreActions =
		{"/journal/rss", "/tags/rss", "/wiki/edit_page_attachment",
		"/message_boards/rss", "/wiki/rss", "/document_library/edit_file_entry",
		"/blogs/rss", "/image_gallery/edit_image"};

	private static final String _PORTAL = "PORTAL";

	private Set<String> _ignoreActions;

}